@@ -6,6 +6,7 @@
 # Code Repository: https://github.com/a-r-j/graphein
 from __future__ import annotations
 
+import logging as log
 import os
 from pathlib import Path
 from typing import Callable, Dict, List, Optional
@@ -16,7 +17,11 @@ from tqdm import tqdm
 from graphein.ml.conversion import GraphFormatConvertor
 from graphein.protein.config import ProteinGraphConfig
 from graphein.protein.graphs import construct_graphs_mp
-from graphein.protein.utils import download_alphafold_structure, download_pdb
+from graphein.protein.utils import (
+    download_alphafold_structure,
+    download_pdb,
+    download_pdb_multiprocessing,
+)
 from graphein.utils.utils import import_message
 
 try:
@@ -137,6 +142,9 @@ class InMemoryProteinGraphDataset(InMemoryDataset):
         elif self.uniprot_ids:
             self.structures = uniprot_ids
         self.af_version = af_version
+        self.bad_pdbs: List[
+            str
+        ] = []  # list of pdb codes that failed to download
 
         # Labels & Chains
         self.graph_label_map = graph_label_map
@@ -172,7 +180,23 @@ class InMemoryProteinGraphDataset(InMemoryDataset):
         """Download the PDB files from RCSB or Alphafold."""
         self.config.pdb_dir = Path(self.raw_dir)
         if self.pdb_codes:
-            [download_pdb(self.config, pdb) for pdb in tqdm(self.pdb_codes)]
+            # Only download PDBs that are not already downloaded
+            to_download = [
+                pdb
+                for pdb in set(self.pdb_codes)
+                if not os.path.exists(Path(self.raw_dir) / f"{pdb}.pdb")
+            ]
+            download_pdb_multiprocessing(
+                to_download,
+                self.raw_dir,
+                max_workers=self.num_cores,
+                strict=False,
+            )
+            self.bad_pdbs = self.bad_pdbs + [
+                pdb
+                for pdb in set(self.pdb_codes)
+                if not os.path.exists(Path(self.raw_dir) / f"{pdb}.pdb")
+            ]
         if self.uniprot_ids:
             [
                 download_alphafold_structure(
@@ -206,6 +230,7 @@ class InMemoryProteinGraphDataset(InMemoryDataset):
 
         # Apply transformations to raw PDB files.
         if self.pdb_transform is not None:
+            print("Transforming PDB files...")
             self.transform_pdbs()
 
         if self.chain_selection_map:
@@ -219,6 +244,7 @@ class InMemoryProteinGraphDataset(InMemoryDataset):
             chain_selections = None
 
         # Create graph objects
+        print("Constructing Graphs...")
         graphs = construct_graphs_mp(
             pdb_path_it=structure_files,
             config=self.config,
@@ -228,32 +254,45 @@ class InMemoryProteinGraphDataset(InMemoryDataset):
         )
         # Transform graphs
         if self.graph_transformation_funcs is not None:
+            print("Transforming Nx Graphs...")
             for func in self.graph_transformation_funcs:
                 graphs = {k: func(v) for k, v in graphs.items()}
 
         # Convert to PyTorch Geometric Data
+        print("Converting Graphs...")
         graphs = {k: self.graph_format_convertor(v) for k, v in graphs.items()}
         graphs = dict(zip(self.structures, graphs.values()))
 
         # Assign labels
         if self.graph_label_map:
+            print("Assigning graph Labels...")
             for k, v in self.graph_label_map.items():
-                graphs[k].graph_y = v
+                try:
+                    graphs[k].graph_y = v
+                except KeyError:
+                    print(f"{k} not found in graphs. Skipping.")
         if self.node_label_map:
+            print("Assigning node Labels...")
             for k, v in self.node_label_map.items():
-                graphs[k].node_y = v
-
+                try:
+                    graphs[k].node_y = v
+                except KeyError:
+                    print(f"{k} not found in graphs. Skipping.")
         data_list = list(graphs.values())
         del graphs
 
         if self.pre_filter is not None:
+            print("Pre-filtering Data...")
             data_list = [g for g in data_list if self.pre_filter(g)]
 
         if self.pre_transform is not None:
+            print("Pre-transforming data...")
             data_list = [self.pre_transform(data) for data in data_list]
 
+        print("Saving Data...")
         data, slices = self.collate(data_list)
         torch.save((data, slices), self.processed_paths[0])
+        print("Done!")
 
 
 class ProteinGraphDataset(Dataset):
@@ -359,6 +398,7 @@ class ProteinGraphDataset(Dataset):
         self.graph_label_map = graph_label_map
         self.node_label_map = node_label_map
         self.chain_selection_map = chain_selection_map
+        self.bad_pdbs: List[str] = []
 
         # Configs
         self.config = graphein_config
@@ -388,7 +428,23 @@ class ProteinGraphDataset(Dataset):
         """Download the PDB files from RCSB or Alphafold."""
         self.config.pdb_dir = Path(self.raw_dir)
         if self.pdb_codes:
-            [download_pdb(self.config, pdb) for pdb in tqdm(self.pdb_codes)]
+            # Only download undownloaded PDBs
+            to_download = [
+                pdb
+                for pdb in set(self.pdb_codes)
+                if not os.path.exists(Path(self.raw_dir) / f"{pdb}.pdb")
+            ]
+            download_pdb_multiprocessing(
+                to_download,
+                self.raw_dir,
+                max_workers=self.num_cores,
+                strict=False,
+            )
+            self.bad_pdbs = self.bad_pdbs + [
+                pdb
+                for pdb in set(self.pdb_codes)
+                if not os.path.exists(Path(self.raw_dir) / f"{pdb}.pdb")
+            ]
         if self.uniprot_ids:
             [
                 download_alphafold_structure(
@@ -442,7 +498,7 @@ class ProteinGraphDataset(Dataset):
                     self.chain_selection_map[pdb]
                     if pdb in self.chain_selection_map.keys()
                     else "all"
-                    for pdb in self.structures
+                    for pdb in chunk
                 ]
             else:
                 chain_selections = None
