@@ -111,13 +111,45 @@ AMINO_ACIDS: List[str] = [
     "Z",
 ]
 """Vocabulary of amino acids with one-letter codes. Includes `fuzzy` standard amino acids:
-``"B"`` denotes ``"ASX"`` which corresponds to ``"ASP"`` (``"D"``) **or** ``"ASN"`` (``"N"``) 
+``"B"`` denotes ``"ASX"`` which corresponds to ``"ASP"`` (``"D"``) **or** ``"ASN"`` (``"N"``),
+``"J"`` denotes ``"XLE"`` which corresponds to ``"LEU"`` (``"L"``) **or** ``"ILE"`` (``"I"``),
 and ``"Z"`` denotes ``"GLX"`` which corresponds to``"GLU"`` (``"E"``) **or** ``"GLN"`` (``"Q"``).
+``"X"`` denotes unknown (``"UNK"`` or sometimes ``"XAA"``).
 """
 
+STANDARD_AMINO_ACID_MAPPING_TO_1_3: Dict[str, str] = {
+    "A": "ALA",
+    "C": "CYS",
+    "D": "ASP",
+    "E": "GLU",
+    "F": "PHE",
+    "G": "GLY",
+    "H": "HIS",
+    "I": "ILE",
+    "K": "LYS",
+    "L": "LEU",
+    "M": "MET",
+    "N": "ASN",
+    "O": "PYL",
+    "P": "PRO",
+    "Q": "GLN",
+    "R": "ARG",
+    "S": "SER",
+    "T": "THR",
+    "U": "SEC",
+    "V": "VAL",
+    "W": "TRP",
+    "Y": "TYR",
+    "X": "UNK",
+}
+
 NON_STANDARD_AMINO_ACID_MAPPING_3_TO_1: Dict[str, str] = {
-    "SEC": "U",
+    "CGU": "E",
+    "HID": "H",
+    "HIE": "H",
+    "HIP": "H",
     "PYL": "O",
+    "SEC": "U",
 }
 """
 Mapping of 3-letter non-standard amino acids codes to their one-letter form.
@@ -135,33 +167,38 @@ BOND_TYPES: List[str] = [
     "cation_pi",
     "backbone",
     "delaunay",
+    "vdw",
+    "vdw_clash",
+    "salt_bridge",
+    "proximal",
+    "bb_carbonyl_carbonyl",
 ]
 """List of supported bond types."""
 
 STANDARD_RESI_NAMES: List[str] = [
     "ALA",
+    "ARG",
+    "ASN",
+    "ASP",
     "ASX",
     "CYS",
-    "ASP",
+    "GLN",
     "GLU",
-    "PHE",
+    "GLX",
     "GLY",
     "HIS",
     "ILE",
-    "LYS",
     "LEU",
+    "LYS",
     "MET",
-    "ASN",
+    "PHE",
     "PRO",
-    "GLN",
-    "ARG",
     "SER",
     "THR",
-    "VAL",
     "TRP",
     "TYR",
-    "GLX",
     "UNK",
+    "VAL",
 ]
 """
 List of standard residue 3-letter names.
@@ -170,23 +207,26 @@ Includes ``"UNK"`` for unknown residues.
 """
 
 NON_STANDARD_RESI_NAMES: List[str] = [
-    "CSD",
-    "HYP",
-    "BMT",
     "5HP",
-    "ACE",
     "ABA",
+    "ACE",
     "AIB",
-    "NH2",
+    "BMT",
+    "BOC",
     "CBX",
+    "CEA",
+    "CGU",
+    "CME",
+    "CRO",
+    "CSD",
+    "CSO",
+    "CSS",
     "CSW",
-    "OCS",
+    "CSX",
+    "CXM",
     "DAL",
     "DAR",
-    "DSG",
-    "DSP",
     "DCY",
-    "CRO",
     "DGL",
     "DGN",
     "DHI",
@@ -196,39 +236,39 @@ NON_STANDARD_RESI_NAMES: List[str] = [
     "DLY",
     "DPN",
     "DPR",
+    "DSG",
     "DSN",
+    "DSP",
     "DTH",
     "DTR",
     "DTY",
     "DVA",
+    "FME",
     "FOR",
-    "CGU",
+    "HID",
+    "HIE",
+    "HIP",
+    "HYP",
     "IVA",
     "KCX",
     "LLP",
-    "CXM",
-    "FME",
     "MLE",
     "MVA",
+    "NH2",
     "NLE",
-    "PTR",
+    "OCS",
     "ORN",
-    "SEP",
-    "SEC",
-    "TPO",
     "PCA",
+    "PTR",
     "PVL",
     "PYL",
     "SAR",
-    "CEA",
-    "CSO",
-    "CSS",
-    "CSX",
-    "CME",
-    "TYS",
-    "BOC",
-    "TPQ",
+    "SEC",
+    "SEP",
     "STY",
+    "TPO",
+    "TPQ",
+    "TYS",
 ]
 """
 List of non-standard residue 3-letter names.
@@ -237,45 +277,32 @@ Collected from: https://www.globalphasing.com/buster/manual/maketnt/manual/lib_v
 """
 
 RESI_NAMES: List[str] = [
-    "ALA",
-    "ASX",
-    "CYS",
-    "ASP",
-    "GLU",
-    "PHE",
-    "GLY",
-    "HIS",
-    "ILE",
-    "LYS",
-    "LEU",
-    "MET",
-    "ASN",
-    "PRO",
-    "GLN",
-    "ARG",
-    "SER",
-    "THR",
-    "VAL",
-    "TRP",
-    "TYR",
-    "GLX",
-    "CSD",
-    "HYP",
-    "BMT",
     "5HP",
-    "ACE",
     "ABA",
+    "ACE",
     "AIB",
-    "NH2",
+    "ALA",
+    "ARG",
+    "ASN",
+    "ASP",
+    "ASX",
+    "BMT",
+    "BOC",
     "CBX",
+    "CEA",
+    "CGU",
+    "CME",
+    "CRO",
+    "CSD",
+    "CSO",
+    "CSS",
     "CSW",
-    "OCS",
+    "CSX",
+    "CXM",
+    "CYS",
     "DAL",
     "DAR",
-    "DSG",
-    "DSP",
     "DCY",
-    "CRO",
     "DGL",
     "DGN",
     "DHI",
@@ -285,40 +312,56 @@ RESI_NAMES: List[str] = [
     "DLY",
     "DPN",
     "DPR",
+    "DSG",
     "DSN",
+    "DSP",
     "DTH",
     "DTR",
     "DTY",
     "DVA",
+    "FME",
     "FOR",
-    "CGU",
+    "GLN",
+    "GLU",
+    "GLX",
+    "GLY",
+    "HID",
+    "HIE",
+    "HIP",
+    "HIS",
+    "HYP",
+    "ILE",
     "IVA",
     "KCX",
+    "LEU",
     "LLP",
-    "CXM",
-    "FME",
+    "LYS",
+    "MET",
     "MLE",
     "MVA",
+    "NH2",
     "NLE",
-    "PTR",
+    "OCS",
     "ORN",
-    "SEP",
-    "SEC",
-    "TPO",
     "PCA",
+    "PHE",
+    "PRO",
+    "PTR",
     "PVL",
     "PYL",
     "SAR",
-    "CEA",
-    "CSO",
-    "CSS",
-    "CSX",
-    "CME",
-    "TYS",
-    "BOC",
-    "TPQ",
+    "SEC",
+    "SEP",
+    "SER",
     "STY",
+    "THR",
+    "TPO",
+    "TPQ",
+    "TRP",
+    "TYR",
+    "TYS",
     "UNK",
+    "VAL",
 ]
 """
 3-letter residue names for all amino acids.
@@ -328,47 +371,34 @@ Includes ``"UNK"`` to denote unknown residues.
 
 # https://www.ebi.ac.uk/pdbe-srv/pdbechem/chemicalCompound
 RESI_THREE_TO_1: Dict[str, str] = {
-    "ALA": "A",
-    "ASX": "B",
-    "CYS": "C",
-    "ASP": "D",
-    "GLU": "E",
-    "PHE": "F",
-    "GLY": "G",
-    "HIS": "H",
-    "ILE": "I",
-    "LYS": "K",
-    "LEU": "L",
-    "MET": "M",
-    "ASN": "N",
-    "PRO": "P",
-    "GLN": "Q",
-    "ARG": "R",
-    "SER": "S",
-    "THR": "T",
-    "VAL": "V",
-    "TRP": "W",
-    "TYR": "Y",
-    "GLX": "Z",
-    "CSD": "C",
-    "HYP": "P",
-    "BMT": "T",
     "3HP": "X",
     "4HP": "X",
     "5HP": "Q",
-    "ACE": "X",
     "ABA": "A",
+    "ACE": "X",
     "AIB": "A",
-    "NH2": "X",
+    "ALA": "A",
+    "ARG": "R",
+    "ASN": "N",
+    "ASP": "D",
+    "ASX": "B",
+    "BMT": "T",
+    "BOC": "X",
     "CBX": "X",
+    "CEA": "C",
+    "CGU": "E",
+    "CME": "C",
+    "CRO": "TYG",
+    "CSD": "C",
+    "CSO": "C",
+    "CSS": "C",
     "CSW": "C",
-    "OCS": "C",
+    "CSX": "C",
+    "CXM": "M",
+    "CYS": "C",
     "DAL": "A",
     "DAR": "R",
-    "DSG": "N",
-    "DSP": "D",
     "DCY": "C",
-    "CRO": "TYG",
     "DGL": "E",
     "DGN": "Q",
     "DHI": "H",
@@ -378,40 +408,56 @@ RESI_THREE_TO_1: Dict[str, str] = {
     "DLY": "K",
     "DPN": "F",
     "DPR": "P",
+    "DSG": "N",
     "DSN": "S",
+    "DSP": "D",
     "DTH": "T",
     "DTR": "W",
     "DTY": "Y",
     "DVA": "V",
+    "FME": "M",
     "FOR": "X",
-    "CGU": "E",
+    "GLN": "Q",
+    "GLU": "E",
+    "GLX": "Z",
+    "GLY": "G",
+    "HID": "H",  # Different protonation states of HIS
+    "HIE": "H",  # Different protonation states of HIS
+    "HIP": "H",  # Different protonation states of HIS
+    "HIS": "H",
+    "HYP": "P",
+    "ILE": "I",
     "IVA": "X",
     "KCX": "K",
+    "LEU": "L",
     "LLP": "K",
-    "CXM": "M",
-    "FME": "M",
+    "LYS": "K",
+    "MET": "M",
     "MLE": "L",
     "MVA": "V",
+    "NH2": "X",
     "NLE": "L",
-    "PTR": "Y",
+    "OCS": "C",
     "ORN": "A",
-    "SEP": "S",
-    "SEC": "U",
-    "TPO": "T",
     "PCA": "Q",
+    "PHE": "F",
+    "PRO": "P",
+    "PTR": "Y",
     "PVL": "X",
     "PYL": "O",
     "SAR": "G",
-    "CEA": "C",
-    "CSO": "C",
-    "CSS": "C",
-    "CSX": "C",
-    "CME": "C",
-    "TYS": "Y",
-    "BOC": "X",
-    "TPQ": "Y",
+    "SEC": "U",
+    "SEP": "S",
+    "SER": "S",
     "STY": "Y",
+    "THR": "T",
+    "TPO": "T",
+    "TPQ": "Y",
+    "TRP": "W",
+    "TYR": "Y",
+    "TYS": "Y",
     "UNK": "X",
+    "VAL": "V",
 }
 """
 Mapping of 3-letter residue names to 1-letter residue names.
@@ -484,23 +530,26 @@ Non-standard residue info taken from: https://www.globalphasing.com/buster/manua
 """
 
 NON_STANDARD_RESIS_PARENT: Dict[str, str] = {
-    "CSD": "CYS",
-    "HYP": "PRO",
-    "BMT": "THR",
     "5HP": "GLU",
-    "ACE": "-",
     "ABA": "ALA",
+    "ACE": "-",
     "AIB": "ALA",
-    "NH2": "-",
+    "BMT": "THR",
+    "BOC": "-",
     "CBX": "-",
+    "CEA": "CYS",
+    "CGU": "GLU",
+    "CME": "CYS",
+    "CRO": "CRO",
+    "CSD": "CYS",
+    "CSO": "CYS",
+    "CSS": "CYS",
     "CSW": "CYS",
-    "OCS": "CYS",
+    "CSX": "CYS",
+    "CXM": "MET",
     "DAL": "ALA",
     "DAR": "ARG",
-    "DSG": "ASN",
-    "DSP": "ASP",
     "DCY": "CYS",
-    "CRO": "CRO",
     "DGL": "GLU",
     "DGN": "GLN",
     "DHI": "HIS",
@@ -510,39 +559,36 @@ NON_STANDARD_RESIS_PARENT: Dict[str, str] = {
     "DLY": "LYS",
     "DPN": "PHE",
     "DPR": "PRO",
+    "DSG": "ASN",
     "DSN": "SER",
+    "DSP": "ASP",
     "DTH": "THR",
     "DTR": "DTR",
     "DTY": "TYR",
     "DVA": "VAL",
+    "FME": "MET",
     "FOR": "-",
-    "CGU": "GLU",
+    "HYP": "PRO",
     "IVA": "-",
     "KCX": "LYS",
     "LLP": "LYS",
-    "PYL": "LYS",
-    "CXM": "MET",
-    "FME": "MET",
     "MLE": "LEU",
     "MVA": "VAL",
+    "NH2": "-",
     "NLE": "LEU",
-    "PTR": "TYR",
+    "OCS": "CYS",
     "ORN": "ALA",
-    "SEP": "SER",
-    "TPO": "THR",
     "PCA": "GLU",
+    "PTR": "TYR",
     "PVL": "-",
+    "PYL": "LYS",
     "SAR": "GLY",
-    "CEA": "CYS",
-    "CSO": "CYS",
-    "CSS": "CYS",
-    "CSX": "CYS",
-    "CME": "CYS",
     "SEC": "CYS",
-    "TYS": "TYR",
-    "BOC": "-",
-    "TPQ": "PHE",
+    "SEP": "SER",
     "STY": "TYR",
+    "TPO": "THR",
+    "TPQ": "PHE",
+    "TYS": "TYR",
 }
 """
 Mapping of 3-letter non-standard/modified residues to their 3-letter parent residue names.
@@ -642,14 +688,14 @@ See http://ligand-expo.rcsb.org/ and https://www.globalphasing.com/buster/manual
 """
 
 CARBOHYDRATE_NAMES: List[str] = [
+    "D-GALACTOSE",
     "D-GLUCOSE",
     "D-MANNOSE",
-    "FUCOSE",
-    "D-GALACTOSE",
-    "N-ACETYL-D-GLUCOSAMINE",
-    "N-ACETYL-D-GALACTOSAMINE",
-    "O-SIALIC_ACID",
     "D-XYLOPYRANOSE",
+    "FUCOSE",
+    "N-ACETYL-D-GALACTOSAMINE",
+    "N-ACETYL-D-GLUCOSAMINE",
+    "O-SIALIC_ACID",
 ]
 """
 Names of commonly found carbohydrates in protein structures.
@@ -659,12 +705,12 @@ See http://ligand-expo.rcsb.org/ and https://www.globalphasing.com/buster/manual
 
 CARBOHYDRATE_CODES: List[str] = [
     "BGC",
-    "GLC",
-    "MAN",
     "BMA",
     "FUC",
     "GAL",
     "GLA",
+    "GLC",
+    "MAN",
     "NAG",
     "NGA",
     "SIA",
@@ -678,12 +724,12 @@ See http://ligand-expo.rcsb.org/ and https://www.globalphasing.com/buster/manual
 
 CARBOHYDRATE_CODE_NAME_MAPPING: Dict[str, str] = {
     "BGC": "D-GLUCOSE",
-    "GLC": "D-GLUCOSE",
-    "MAN": "D-MANNOSE",
     "BMA": "D-MANNOSE",
     "FUC": "FUCOSE",
     "GAL": "D-GALACTOSE",
     "GLA": "D-GALACTOSE",
+    "GLC": "D-GLUCOSE",
+    "MAN": "D-MANNOSE",
     "NAG": "N-ACETYL-D-GLUCOSAMINE",
     "NGA": "N-ACETYL-D-GALACTOSAMINE",
     "SIA": "O-SIALIC_ACID",
@@ -697,14 +743,14 @@ See http://ligand-expo.rcsb.org/ and https://www.globalphasing.com/buster/manual
 
 HYDROPHOBIC_RESIS: List[str] = [
     "ALA",
-    "VAL",
-    "LEU",
     "ILE",
+    "LEU",
     "MET",
     "PHE",
-    "TRP",
     "PRO",
+    "TRP",
     "TYR",
+    "VAL",
 ]
 """List of residues that are considered to be hydrophobic."""
 
@@ -724,15 +770,26 @@ NEG_AA: List[str] = ["GLU", "ASP"]
 """Negatively charged amino acids."""
 
 AA_RING_ATOMS: Dict[str, List[str]] = {
-    "PHE": ["CG", "CD", "CE", "CZ"],
-    "TRP": ["CD", "CE", "CH", "CZ"],
-    "HIS": ["CG", "CD", "CE", "ND", "NE"],
-    "TYR": ["CG", "CD", "CE", "CZ"],
+    # "HIS": ["CG", "CD", "CE", "ND", "NE"],
+    "HIS": ["CG", "CD", "CE", "ND", "NE", "CD2", "ND1", "CE1", "NE2"],
+    # "PHE": ["CG", "CD", "CE", "CZ"],
+    "PHE": ["CG", "CD", "CE", "CZ", "CD1", "CD2", "CE1", "CE2"],
+    # "TRP": ["CD", "CE", "CH", "CZ"],
+    "TRP": ["CD2", "CE2", "CE3", "CZ2", "CZ3", "CH2"],
+    # "TYR": ["CG", "CD", "CE", "CZ"],
+    "TYR": ["CG", "CD", "CE", "CZ", "CD1", "CD2", "CE1", "CE2"],
 }
 """
 Dictionary mapping amino acid 3-letter codes to lists of atoms that are part of rings.
 """
 
+RING_NORMAL_ATOMS: Dict[str, List[str]] = {
+    "PHE": ["CG", "CE1", "CE2"],
+    "TRP": ["CD2", "CZ2", "CZ3"],
+    "TYR": ["CG", "CE1", "CE2"],
+}
+"""Dictionary of atoms used to compute ring normals for each residue."""
+
 AROMATIC_RESIS: List[str] = ["PHE", "TRP", "HIS", "TYR"]
 """List of aromatic residues."""
 
@@ -748,14 +805,48 @@ PI_RESIS: List[str] = ["PHE", "TYR", "TRP"]
 SULPHUR_RESIS: List[str] = ["MET", "CYS"]
 """Residues containing sulphur atoms."""
 
+SALT_BRIDGE_ANIONS: List[str] = ["ASP", "GLU"]
+"""List of anionic residues that can form salt bridges."""
+
+SALT_BRIDGE_CATIONS: List[str] = ["LYS", "ARG"]
+"""List of cationic residues that can form salt bridges."""
+
+SALT_BRIDGE_RESIDUES: List[str] = SALT_BRIDGE_ANIONS + SALT_BRIDGE_CATIONS
+"""List of residues that can form salt bridges."""
+
+SALT_BRIDGE_ATOMS: List[str] = ["OD1", "OD2", "OE1", "OE2", "NZ", "NH1", "NH2"]
+"""List of atoms that can form salt bridges."""
+
+VDW_RADII: Dict[str, float] = {
+    "H": 1.2,  # 1.09
+    "C": 1.7,
+    "N": 1.55,
+    "O": 1.52,
+    "F": 1.47,
+    "P": 1.8,
+    "S": 1.8,
+    "Cl": 1.75,
+    "Cu": 1.4,
+}
+"""van der Waals radii of the most common atoms. Taken from:
+
+> Bondi, A. (1964). "van der Waals Volumes and Radii".
+> J. Phys. Chem. 68 (3): 441–451.
+
+https://pubs.acs.org/doi/10.1021/j100785a001
+"""
+
+
 ISOELECTRIC_POINTS: Dict[str, float] = {
     "ALA": 6.11,
     "ARG": 10.76,
     "ASN": 10.76,
     "ASP": 2.98,
+    "ASX": 6.87,  # the average of D and N
     "CYS": 5.02,
-    "GLU": 3.08,
     "GLN": 5.65,
+    "GLU": 3.08,
+    "GLX": 4.35,  # the average of E and Q
     "GLY": 6.06,
     "HIS": 7.64,
     "ILE": 6.04,
@@ -768,10 +859,8 @@ ISOELECTRIC_POINTS: Dict[str, float] = {
     "THR": 5.60,
     "TRP": 5.88,
     "TYR": 5.63,
-    "VAL": 6.02,
     "UNK": 7.00,  # unknown so assign neutral
-    "ASX": 6.87,  # the average of D and N
-    "GLX": 4.35,  # the average of E and Q
+    "VAL": 6.02,
 }
 """
 Dictionary of isoelectric points for standard amino acids. For ``"UNK"`` residues, neutral (pH 7.0) is assigned.
@@ -797,9 +886,11 @@ MOLECULAR_WEIGHTS: Dict[str, float] = {
     "ARG": 174.2017,
     "ASN": 132.1184,
     "ASP": 133.1032,
+    "ASX": 132.6108,  # the average of D and N
     "CYS": 121.1590,
-    "GLU": 147.1299,
     "GLN": 146.1451,
+    "GLU": 147.1299,
+    "GLX": 146.6375,  # the average of E and Q
     "GLY": 75.0669,
     "HIS": 155.1552,
     "ILE": 131.1736,
@@ -812,12 +903,10 @@ MOLECULAR_WEIGHTS: Dict[str, float] = {
     "THR": 119.1197,
     "TRP": 204.2262,
     "TYR": 181.1894,
-    "VAL": 117.1469,
     "UNK": 137.1484,  # unknown, therefore assign average of knowns
-    "ASX": 132.6108,  # the average of D and N
-    "GLX": 146.6375,  # the average of E and Q
+    "VAL": 117.1469,
 }
-"""Mapping of 3-letter amino acid names to molecular weights. 
+"""Mapping of 3-letter amino acid names to molecular weights.
 ``UNK`` is used for unknown residues and takes the mean of known weights.
 For ``"ASX"`` and ``"GLX"`` the average of their constituents (``"D"`` and ``"N"``, and ``"E"`` and ``"Q"``, respectively) is assigned.
 """
@@ -828,7 +917,7 @@ MOLECULAR_WEIGHTS_STD = {
     for k, v in MOLECULAR_WEIGHTS.items()
 }
 """
-Standardized (sklearn.StandardScaler) molecular weights for standard amino acids. 
+Standardized (sklearn.StandardScaler) molecular weights for standard amino acids.
 
 See :const:`~graphein.protein.resi_atoms.MOLECULAR_WEIGHTS` for details.
 """
@@ -1914,3 +2003,13 @@ Taken from:
     Jon C. Baber and Edward E. Hodgkin*
     J. Chem. Inf. Comput. Sci. 1992, 32. 401-406
 """
+
+ATOMIC_MASSES: Dict[str, float] = {
+    "C": 12.0107,
+    "H": 1.00794,
+    "O": 15.9994,
+    "N": 14.0067,
+    "P": 30.9738,
+    "S": 32.065,
+}
+"""Dictionary of atomic masses for standard protein elements."""
